var Figure = function () {
    this.globalAlpha = 1.0;
    this.globalCompositeOperation = "source-over";
    this.strokeStyle = "black";
    this.fillStyle = "black";
    this.shadowOffsetX = 0.0;
    this.shadowOffsetY = 0.0;
    this.shadowBlur = 0.0;
    this.shadowColor = "black";
    this.lineWidth = 1.0;
    this.lineCap = "butt";
    this.lineJoin = "miter";
    this.miterLimit = 10.0;
    this.lineDashOffset = 5.0;
    this.lineDashSegments = [];
    this.fill = false;
    this.stroke = false;
};
Figure.prototype.setProps = function (context) {
    context.globalAlpha = this.globalAlpha;
    context.globalCompositeOperation = this.globalCompositeOperation;
    context.strokeStyle = this.strokeStyle;
    context.fillStyle = this.fillStyle;
    context.shadowOffsetX = this.shadowOffsetX;
    context.shadowOffsetY = this.shadowOffsetY;
    context.shadowBlur = this.shadowBlur;
    context.shadowColor = this.shadowColor;
    context.lineWidth = this.lineWidth;
    context.lineCap = this.lineCap;
    context.lineJoin = this.lineJoin;
    context.miterLimit = this.miterLimit;
    context.lineDashOffset = this.lineDashOffset;
    context.lineDashSegments = this.lineDashSegments;
};
// Point
// 0 - tworzy punkt P(0,0)
// 2 - dwie współrzędne x i y
var Point = function () {
    var len = arguments.length;
    if (len === 0) {
        this.x = 0.0;
        this.y = 0.0;
    } else if (len === 2) {
        this.x = arguments[0];
        this.y = arguments[1];
    }
    this.diam = 5.0;// promień
    this.shape = "circle";
};
Point.prototype = new Figure();
Point.prototype.distance = function (point) {
    var a = point.x - this.x;
    var b = point.y - this.y;
    return Math.sqrt(a * a + b * b);
};
Point.prototype.deepclone = function () {
    var figure = new Point();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.x = this.x;
    figure.y = this.y;
    figure.diam = this.diam;
    figure.shape = this.shape;
    return figure;
};
Point.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    switch (this.shape) {
        case "circle":
            context.arc(this.x, this.y, this.diam / 2.0, 0.0, 2 * Math.PI, false);
            break;
        case "square":
            context.rect(this.x - this.diam / 2.0, this.y - this.diam / 2.0,
                    this.diam, this.diam);
            break;
        case "cross":
            context.moveTo(this.x - this.diam / 2.0, this.y);
            context.lineTo(this.x + this.diam / 2.0, this.y);
            context.moveTo(this.x, this.y - this.diam / 2.0);
            context.lineTo(this.x, this.y + this.diam / 2.0);
            break;
    }
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// Segment = Odcinek
// Liczba argumentów
// 0 - automatyczne wstawienie tego samego punktu (0,0)
// 1 - tablica punktów
// 2 - dwa punkty
// 4 - cztery wspolrzedne dwoch punktow (x,y)
var Segment = function () {
    this.points = [];
    var len = arguments.length;
    if (len === 0) {
        this.points[0] = new Point(0.0, 0.0);
        this.points[1] = new Point(0.0, 0.0);
    } else if (len === 1) {
        for (var i = 0; i < 2; i++) {
            this.points[i] = new Point(arguments[0][i].x, arguments[0][i].y);
        }
    } else if (len === 2) {
        this.points[0] = new Point(arguments[0].x, arguments[0].y);
        this.points[1] = new Point(arguments[1].x, arguments[1].y);
    } else if (len === 4) {
        this.points[0] = new Point(arguments[0], arguments[1]);
        this.points[1] = new Point(arguments[2], arguments[3]);
    }
};
Segment.prototype = new Figure();
Segment.prototype.length = function () {
    return distance(this.points[0], this.points[1]);
};
Segment.prototype.deepclone = function () {
    var figure = new Segment();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    return figure;
};
Segment.prototype.barycenter = function () {
    var x = (this.points[0].x + this.points[1].x) / 2.0;
    var y = (this.points[0].y + this.points[1].y) / 2.0;
    return new Point(x, y);
};
Segment.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    context.moveTo(this.points[0].x, this.points[0].y);
    context.lineTo(this.points[1].x, this.points[1].y);
    if (this.stroke === true) {
        context.stroke();
    }
    context.restore();
};
function distance(pointA, pointB) {
    var a = pointB.x - pointA.x;
    var b = pointB.y - pointA.y;
    return Math.sqrt(a * a + b * b);
}
;
// Triangle (trójkąt)
// Liczba argumentów
// 0 - automatyczne wstawienie tego samego punktu (0,0)
// 1 - tablica punktów
// 3 - trzy punkty
// 6 - sześć wspolrzednych trzech punktow (x,y)
var Triangle = function () {
    this.points = [];
    var len = arguments.length;
    if (len === 0) {
        this.points[0] = new Point(0.0, 0.0);
        this.points[1] = new Point(0.0, 0.0);
        this.points[2] = new Point(0.0, 0.0);
    } else if (len === 1) {
        for (var i = 0; i < 3; i++) {
            this.points[i] = new Point(arguments[0][i].x, arguments[0][i].y);
        }
    } else if (len === 3) {
        this.points[0] = new Point(arguments[0].x, arguments[0].y);
        this.points[1] = new Point(arguments[1].x, arguments[1].y);
        this.points[2] = new Point(arguments[2].x, arguments[2].y);
    } else if (len === 6) {
        this.points[0] = new Point(arguments[0], arguments[1]);
        this.points[1] = new Point(arguments[2], arguments[3]);
        this.points[2] = new Point(arguments[4], arguments[5]);
    }
};
Triangle.prototype = new Figure();
Triangle.prototype.isTriangle = function () {
    var d1 = distance(this.points[0], this.points[1]);
    var d2 = distance(this.points[0], this.points[2]);
    var d3 = distance(this.points[1], this.points[2]);
    var tab = new Array(d1, d2, d3);
    tab.sort(comparison);
    if (tab[0] < tab[1] + tab[2]) {
        return true;
    }
    return false;
};
Triangle.prototype.area = function () {
    var a = distance(this.points[0], this.points[1]);
    var b = distance(this.points[0], this.points[2]);
    var c = distance(this.points[1], this.points[2]);
    var p = (a + b + c) / 2.0;
    return Math.sqrt(p * (p - a) * (p - b) * (p - c));
};
Triangle.prototype.perimeter = function () {
    var a = distance(this.points[0], this.points[1]);
    var b = distance(this.points[0], this.points[2]);
    var c = distance(this.points[1], this.points[2]);
    return a + b + c;
};
Triangle.prototype.deepclone = function () {
    var figure = new Triangle();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    return figure;
};
Triangle.prototype.barycenter = function () {
    var x = (this.points[0].x + this.points[1].x + this.points[2].x) / 3.0;
    var y = (this.points[0].y + this.points[1].y + this.points[2].y) / 3.0;
    return new Point(x, y);
};
Triangle.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.lineCap = "square";
    this.setProps(context);
    context.moveTo(this.points[0].x, this.points[0].y);
    context.lineTo(this.points[1].x, this.points[1].y);
    context.lineTo(this.points[2].x, this.points[2].y);
    context.closePath();
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// Rectangle (Prostokąt)
// Liczba argumentów
// 0 - automatyczne wstawienie tego samego punktu (0,0)
// 1 - tablica punktów
// 3 - Point(x, y), w, h
// 4 - x, y, w, h
var Rectangle = function () {
    this.points = [];
    var len = arguments.length;
    if (len === 0) {
        this.points[0] = new Point(0.0, 0.0);
        this.points[1] = new Point(0.0, 0.0);
        this.points[2] = new Point(0.0, 0.0);
        this.points[3] = new Point(0.0, 0.0);
    } else if (len === 1) {
        for (var i = 0; i < 4; i++) {
            this.points[i] = new Point(arguments[0][i].x, arguments[0][i].y);
        }
    } else if (len === 3) {
        this.points[0] = new Point(arguments[0].x, arguments[0].y);
        this.points[1] = new Point(arguments[0].x + arguments[1],
                arguments[0].y);
        this.points[2] = new Point(arguments[0].x + arguments[1],
                arguments[0].y + arguments[2]);
        this.points[3] = new Point(arguments[0].x, arguments[0].y
                + arguments[2]);

    } else if (len === 4) {
        this.points[0] = new Point(arguments[0], arguments[1]);
        this.points[1] = new Point(arguments[0] + arguments[2], arguments[1]);
        this.points[2] = new Point(arguments[0] + arguments[2], arguments[1]
                + arguments[3]);
        this.points[3] = new Point(arguments[0], arguments[1] + arguments[3]);
    }
};
Rectangle.prototype = new Figure();
Rectangle.prototype.isRectangle = function () {
    var d1 = roundToDecimal(distance(this.point[0], this.points[1]), 4);
    var d2 = roundToDecimal(distance(this.points[1], this.points[2]), 4);
    var d3 = roundToDecimal(distance(this.points[2], this.points[3]), 4);
    var d4 = roundToDecimal(distance(this.points[3], this.points[0]), 4);
    if (d1 === d3 && d2 === d4) {
        return true;
    }
    return false;
};
Rectangle.prototype.area = function () {
    var a = distance(this.points[0], this.points[1]);
    var b = distance(this.points[1], this.points[2]);
    return a * b;
};
Rectangle.prototype.deepclone = function () {
    var figure = new Rectangle();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    return figure;
};
Rectangle.prototype.barycenter = function () {
    var x = (this.points[0].x + this.points[1].x) / 2.0;
    var y = (this.points[0].y + this.points[3].y) / 2.0;
    return new Point(x, y);
};
Rectangle.prototype.perimeter = function () {
    var a = distance(this.points[0], this.points[1]);
    var b = distance(this.points[1], this.points[2]);
    return 2 * (a + b);
};
Rectangle.prototype.diagonal = function () {
    var a = distance(this.points[0], this.points[1]);
    var b = distance(this.points[1], this.points[2]);
    return Math.sqrt(a * a + b * b);
};
Rectangle.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    context.moveTo(this.points[0].x, this.points[0].y);
    context.lineTo(this.points[1].x, this.points[1].y);
    context.lineTo(this.points[2].x, this.points[2].y);
    context.lineTo(this.points[3].x, this.points[3].y);
    context.closePath();
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// IrregPolygon
// Liczba argumentów
// 0 Tworzy wielokąt zerowy
// 1 - tablica punktów
// więcej niż 1 - pojedyncze punkty Point(x,y)
var IrregPolygon = function () {
    this.points = [];
    var len = arguments.length;
    if (len === 0) {
        this.points[0] = new Point(0.0, 0.0);
    } else if (len === 1) {
        for (var i = 0; i < arguments[0].length; i++) {
            this.points[i] = new Point(arguments[0][i].x, arguments[0][i].y);
        }
    } else {
        for (var i = 0; i < len; i++) {
            this.points[i] = new Point(arguments[i].x, arguments[i].y);
        }
    }
};
IrregPolygon.prototype = new Figure();
IrregPolygon.prototype.deepclone = function () {
    var figure = new IrregPolygon();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    return figure;
};
IrregPolygon.prototype.barycenter = function () {
    var cv1 = document.getElementById("testcanvas");
    var ctx1 = cv1.getContext("2d");
    var w = cv1.width;
    var h = cv1.height;
    var barray = this.deepclone();
    var len = barray.points.length;
    barray.draw(ctx1);
    var last;
    var triangle;
    var segment;
    var centrum;
    var area = 0.0;
    var areax = 0.0;
    var areay = 0.0;
    while (len > 2) {
        last = len - 1;
        triangle = new Triangle(barray.points[0], barray.points[1],
                barray.points[last]);
        segment = new Segment(barray.points[1], barray.points[last]);
        centrum = segment.barycenter();
        if (ctx1.isPointInPath(centrum.x, centrum.y)) {
            var arear = triangle.area();
            area += arear;
            var bary = triangle.barycenter();
            areax += (bary.x * arear);
            areay += (bary.y * arear);
            barray.points.shift();
            len = barray.points.length;

        } else {
            triangle = new Triangle(barray.points[last], barray.points[0],
                    barray.points[last - 1]);
            segment = new Segment(barray.points[0], barray.points[last - 1]);
            centrum = segment.barycenter();
            var arear = triangle.area();
            area += arear;
            var bary = triangle.barycenter();
            areax += (bary.x * arear);
            areay += (bary.y * arear);
            barray.points.pop();
            len = barray.points.length;
        }
    }
    return new Point(areax / area, areay / area);
};
IrregPolygon.prototype.area = function () {
    var cv1 = document.getElementById("testcanvas");
    var ctx1 = cv1.getContext("2d");
    var w = cv1.width;
    var h = cv1.height;
    var barray = this.deepclone();
    var len = barray.points.length;
    barray.draw(ctx1);
    var last;
    var triangle;
    var segment;
    var centrum;
    var area = 0.0;
    while (len > 2) {
        last = len - 1;
        triangle = new Triangle(barray.points[0], barray.points[1],
                barray.points[last]);
        segment = new Segment(barray.points[1], barray.points[last]);
        centrum = segment.barycenter();
        if (ctx1.isPointInPath(centrum.x, centrum.y)) {
            var arear = triangle.area();
            area += arear;
            barray.points.shift();
            len = barray.points.length;
        } else {
            triangle = new Triangle(barray.points[last], barray.points[0],
                    barray.points[last - 1]);
            segment = new Segment(barray.points[0], barray.points[last - 1]);
            var arear = triangle.area();
            area += arear;
            barray.points.pop();
            len = barray.points.length;
        }
    }
    return area;
};
IrregPolygon.prototype.perimeter = function () {
    var len = this.points.length;
    var perimeter = distance(this.points[len - 1], this.points[0]);
    for (var i = 0; i < len - 1; i++) {
        perimeter += distance(this.points[i], this.points[i + 1]);
    }
    return perimeter;
};
IrregPolygon.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    context.moveTo(this.points[0].x, this.points[0].y);
    for (var i = 1; i < this.points.length; i++) {
        context.lineTo(this.points[i].x, this.points[i].y);
    }
    context.closePath();
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// RegPolygon (Wielobok)
// Liczba argumentów
// 0 Tworzy wielokąt z jednym punktem P(0,0) i
// promieniu (0,0)
// 3 środek P(x,y), liczba boków, promień;
var RegPolygon = function () {
    this.points = [];
    var len = arguments.length;
    if (len === 0) {
        this.center = new Point(0, 0);
        this.sides = 0;
        this.r = 0;
        this.points[0] = new Point(0.0, 0.0);
    } else if (len === 3) {
        this.center = new Point(arguments[0].x, arguments[0].y);
        this.sides = arguments[1];// liczba boków
        this.r = arguments[2];// promień
        this.points[0] = new Point(arguments[0].x + this.r, arguments[0].y);
        for (var i = 1; i < this.sides; i++) {
            this.points[i] = new Point(arguments[0].x + this.r
                    * Math.cos(i * 2 * Math.PI / this.sides), arguments[0].y
                    + this.r * Math.sin(i * 2 * Math.PI / this.sides));
        }
    }
};
RegPolygon.prototype = new Figure();
RegPolygon.prototype.area = function () {
    var area = 0.0;
    var a = distance(this.points[0], this.points[1]);
    var ka = a * a;
    switch (this.sides) {
        case 3:
            area = Math.sqrt(3) / 4.0 * ka;
            break;
        case 4:
            area = ka;
            break;
        case 5:
            area = Math.sqrt(25 + 10 * Math.sqrt(5)) / 4 * ka;
            break;
        case 6:
            area = 3 * Math.sqrt(3) / 2 * ka;
            break;
        case 7:
            area = 3.6339 * ka;
            break;
        case 8:
            area = 2(1 + Math.sqrt(2)) * ka;
            break;
        case 9:
            area = 6.1818 * ka;
            break;
        case 10:
            area = 5 * Math.sqrt(25 + 10 * Math.sqrt(5)) / 2 * ka;
            break;
        case 12:
            area = 3 * (2 + Math.sqrt(3)) * ka;
            break;
        default:
            area = this.sides / 4
                    * (Math.cos(Math.PI / sides) / Math.sin(Math.PI / sides)) * ka;
            break;
    }
    return area;
};
RegPolygon.prototype.deepclone = function () {
    var figure = new RegPolygon();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    figure.center = new Point(this.center.x, this.center.y);
    figure.sides = this.sides;
    figure.r = this.r;
    return figure;
};
RegPolygon.prototype.barycenter = function () {
    return new Point(this.center.x, this.center.y);
};
RegPolygon.prototype.perimeter = function () {
    var dist = distance(this.points[0], this.points[1]);
    return dist * this.sides;
};
RegPolygon.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    context.moveTo(this.points[0].x, this.points[0].y);
    for (var i = 1; i < this.sides; i++) {
        context.lineTo(this.points[i].x, this.points[i].y);
    }
    context.closePath();
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// StarPolygon - wielokąt gwiaździsty
// Liczba argumentów
// 0 Tworzy wielokąt zerowy z jednym punktem P(0,0) i //promieniu (0,0)
// 4 środek P(x,y), liczba boków, co ile, promień;
var StarPolygon = function () {
    this.points = [];
    var len = arguments.length;
    if (len === 0) {
        this.center = new Point(0, 0);// środek wielokąta
        this.sides = 0;// liczba boków
        this.k = 0; // co ile wierzchołków
        this.r = 0; // promień okręgu opisanego
        this.points[0] = new Point(0.0, 0.0);
    } else if (len === 4) {
        this.center = new Point(arguments[0].x, arguments[0].y);
        this.sides = arguments[1];// liczba boków
        this.k = arguments[2];
        this.r = arguments[3];// promień okręgu opisanego
        this.points[0] = new Point(arguments[0].x + this.r, arguments[0].y);
        for (var i = 1; i < this.sides; i++) {
            this.points[i] = new Point(arguments[0].x + this.r
                    * Math.cos(i * 2 * Math.PI / this.sides), arguments[0].y
                    + this.r * Math.sin(i * 2 * Math.PI / this.sides));
        }
    }
};
StarPolygon.prototype = new Figure();
StarPolygon.prototype.deepclone = function () {
    var figure = new StarPolygon();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    figure.center = new Point(this.center.x, this.center.y);
    figure.sides = this.sides;
    figure.r = this.r;
    figure.k = this.k;
    return figure;
};
StarPolygon.prototype.barycenter = function () {
    return new Point(this.center.x, this.center.y);
};
StarPolygon.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    var n = this.points.length;
    var k = this.k;
    var ilefigur = nwd2(n, k);
    var ileelem = n / ilefigur;
    var points1 = sortCyclic(this.points, k);
    if (ilefigur === 1) {
        context.moveTo(points1[0].x, points1[0].y);
        for (var i = 1; i < n; i++) {
            context.lineTo(points1[i].x, points1[i].y);
        }
        context.closePath();
    } else {
        for (var j = 0; j < n; j += ileelem) {
            context.moveTo(points1[j].x, points1[j].y);
            for (var m = j; m < j + ileelem - 1; m++) {
                context.lineTo(points1[m + 1].x, points1[m + 1].y);
            }
            context.closePath();
        }
    }
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// Star - gwiazda
// Liczba argumentów
// Gwiazda zerowa
// 4 środek P(x,y), outerR, innerR, apices;
var Star = function () {
    this.points = [];
    this.points1 = [];
    var len = arguments.length;
    if (len === 0) {
        this.center = new Point(0, 0);// środek wielokąta
        this.outerR = 0.0;// średnica zewnętrzna
        this.innerR = 0.0; // średnica wewnętrzna
        this.apices = 0; // liczba promieni (wierzchołków)
        this.points[0] = new Point(0.0, 0.0);
        this.alfa = 0.0;
    } else if (len === 4) {
        this.center = new Point(arguments[0].x, arguments[0].y);
        this.outerR = arguments[1];// promień okręgu opisanego
        this.innerR = arguments[2];// promień okręgu wpisanego
        this.apices = arguments[3];// liczba promieni (wierzchołków)
        this.alfa = 360.0 / this.apices;
        var r = 0.5 * this.outerR;
        for (var i = 0; i < this.apices; i++) {
            this.points[i] = new Point(this.center.x + r
                    * Math.cos(degToRad(this.alfa * i)), this.center.y + r
                    * Math.sin(degToRad(this.alfa * i)));
            this.points1[i] = new Point(this.center.x + this.innerR / 2.0
                    * Math.cos(degToRad((this.alfa * i) + 0.5 * this.alfa)),
                    this.center.y
                    + this.innerR
                    / 2.0
                    * Math.sin(degToRad((this.alfa * i) + 0.5
                            * this.alfa)));
        }
    }
};
Star.prototype = new Figure();
Star.prototype.deepclone = function () {
    var figure = new StarPolygon();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.points = [];
    for (var i = 0; i < this.points.length; i++) {
        figure.points[i] = new Point(this.points[i].x, this.points[i].y);
    }
    figure.points1 = [];
    for (var i = 0; i < this.points1.length; i++) {
        figure.points1[i] = new Point(this.points1[i].x, this.points1[i].y);
    }
    figure.center = new Point(this.center.x, this.center.y);
    figure.outerR = this.outerR;
    figure.innerR = this.innerR;
    figure.apices = this.apices;
    figure.alfa = this.alfa;
    return figure;
};
Star.prototype.barycenter = function () {
    return new Point(this.center.x, this.center.y);
};
Star.prototype.draw = function (context) {
    context.save();
    context.beginPath();
    this.setProps(context);
    context.moveTo(this.points1[this.apices - 1].x,
            this.points1[this.apices - 1].y);
    for (var i = this.apices - 1; i > 0; i--) {
        context.lineTo(this.points[i].x, this.points[i].y);
        context.lineTo(this.points1[i - 1].x, this.points1[i - 1].y);
    }
    context.lineTo(this.points[0].x, this.points[0].y);
    context.closePath();
    if (this.stroke === true) {
        context.stroke();
    }
    if (this.fill === true) {
        context.fill();
    }
    context.restore();
};
// Oval
// Liczba argumentów
// 0 - owal zerowy
// 4 punkt centralny Point(x,y), promień, startAngle, endAngle
// 5 punkt centralny Point(x,y), promień, startAngle, endAngle, połowa wysokości
// elipsy
// jeśli to koło to parametr jest równy promieniowi
// 6 punkt centralny Point(x,y), promień, startAngle, endAngle, połowa wysokości
// elipsy,
// jeśli to koło to parametr jest równy promieniowi
// czy to jest odcinek czy wycinek (true - odcinek koła lub elipsy,
// false (domyślny) - wycinek koła lub elipsy)
var Oval = function () {
    var len = arguments.length;
    this.counterClockwise = true;
    if (len === 0) {
        this.center = new Point(0, 0);
        this.r = 0.0;
        this.startAngle = 0.0;
        this.endAngle = 0.0;
        this.b = this.r;
        this.segment = false;
    } else if (len === 4) {
        this.center = new Point(arguments[0].x, arguments[0].y);
        this.r = arguments[1];
        this.startAngle = arguments[2];
        this.endAngle = arguments[3];
        this.b = this.r;
        this.segment = false;
    } else if (len === 5) {
        this.center = new Point(arguments[0].x, arguments[0].y);
        this.r = arguments[1];
        this.startAngle = arguments[2];
        this.endAngle = arguments[3];
        this.b = arguments[4];
        this.segment = false;
    } else if (len === 6) {
        this.center = new Point(arguments[0].x, arguments[0].y);
        this.r = arguments[1];
        this.startAngle = arguments[2];
        this.endAngle = arguments[3];
        this.b = arguments[4];
        this.segment = arguments[5];
    }
};
Oval.prototype = new Figure();
Oval.prototype.deepclone = function () {
    var figure = new StarPolygon();
    figure.globalAlpha = this.globalAplpha;
    figure.globalCompositeOperation = this.globalCompositeOperation;
    figure.strokeStyle = this.strokeStyle;
    figure.fillStyle = this.fillStyle;
    figure.shadowOffsetX = this.shadowOffsetX;
    figure.shadowOffsetY = this.shadowOffsetY;
    figure.shadowBlur = this.shadowBlur;
    figure.shadowColor = this.shadowColor;
    figure.lineWidth = this.lineWidth;
    figure.lineCap = this.lineCap;
    figure.lineJoin = this.lineJoin;
    figure.miterLimit = this.miterLimit;
    figure.lineDashOffset = this.lineDashOffset;
    figure.lineDashSegments = [];
    for (var i = 0; i < this.lineDashSegments.length; i++) {
        figure.lineDashSegments[i] = this.lineDashSegments[i];
    }
    figure.fill = this.fill;
    figure.stroke = this.stroke;
    figure.center = new Point(this.center.x, this.center.y);
    figure.r = this.r;
    figure.startAngle = this.startAngle;
    figure.endAngle = this.endAngle;
    figure.counterClockwise = this.counterClockwise;
    figure.b = this.b;
    figure.segment = this.segment;
    return figure;
};
Oval.prototype.barycenter = function () {
    return new Point(this.center.x, this.center.y);
};
Oval.prototype.draw = function (context) {
    var angle = 2 * Math.PI;
    context.save();
    context.beginPath();
    this.setProps(context);
    var s = this.b / this.r;
    context.translate(0, this.center.y - this.center.y * s);
    context.scale(1, s);
    if (this.stroke === true) {
        context
                .arc(this.center.x, this.center.y, this.r, angle
                        - this.startAngle, angle - this.endAngle,
                        this.counterClockwise);
        context.stroke();
    }
    if (this.fill === true) {
        if (this.segment) {
            context
                    .arc(this.center.x, this.center.y, this.r, angle
                            - this.startAngle, angle - this.endAngle,
                            this.counterClockwise);
        } else {

            context
                    .arc(this.center.x, this.center.y, this.r, angle
                            - this.startAngle, angle - this.endAngle,
                            this.counterClockwise);
            context.lineTo(this.center.x, this.center.y);

        }
        context.fill();
    }
    context.restore();
};
// ---------------------- różne funkcje pomocnicze ---------------------------
function comparison(x, y) {
    return y - x;
};
function roundToDecimal(num, dec) {
    var multi = Math.round(Math.pow(10, dec));
    var temp = Math.round(num * multi);
    var temp1 = temp * 1.0 / multi;
    return temp1;
};
function deepclone(parray) {
    var arraynew = [];
    for (var i = 0; i < parray.length; i++) {
        arraynew[i] = new Point(parray[i].x, parray[i].y);
    }
    return arraynew;
};
function nwd2(x, y) {
    if (y === 0) {
        return x;
    }
    while (x !== y) {
        if (x > y)
            x -= y;
        else
            y -= x;
    }
    return x;
};
function cutDecimal(nr) {
    var temp = nr.toString();
    var temp1 = temp.indexOf(".");
    var temp2 = "";
    if (temp1 > -1) {
        temp2 = temp.substr(0, temp1);
    } else {
        temp2 = temp;
    }
    return parseInt(temp2);
};
// k = krok )(co ile);
// zwraca nową tablicę liczb posortowanych cyklicznie
function sortCyclic(array, k) {
    var n = array.length;
    var tabl = [];
    var nwd = nwd2(n, k);
    var m = 0;
    switch (nwd) {
        case 1:
            var h = n * k;
            for (var i = 0; i < h; i += k) {
                tabl[m] = array[i - cutDecimal(i / n) * n];
                m++;
            }
            break;
        default:
            var h1 = n * k / nwd;
            for (var j = 0; j < nwd; j++) {
                for (var i = j; i < h1; i += k) {
                    tabl[m] = array[i - cutDecimal(i / n) * n];
                    m++;
                }
            }
            break;
    }
    return tabl;
};
var degToRad = function (deg) {
    return Math.PI * deg / 180.0;
};